# ==================================================================================================
# Copyright 2013 Twitter, Inc.
# --------------------------------------------------------------------------------------------------
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==================================================================================================

import socket
import ssl

from thrift.transport.TSSLSocket import TSSLSocket
from thrift.transport.TTransport import TTransportException


class DelayedHandshakeTSSLSocket(TSSLSocket):
  """Monkeypatched TSSLSocket that allows delaying of the initial SSL handshake.

  The purpose of this DelayedHandshakeTSSLSocket is to allow for intermixing other
  transport-layer protocols such as SOCKS.

  Behaves the same as TSSLSocket except it accepts the delay_handshake
  keyword argument.  This defers the SSL handshake until *after* connect.
  """

  def __init__(self, *args, **kw):
    # Curse 2.6.x + PEP-3102
    self.__delay_handshake = kw.pop('delay_handshake', False)
    self.__socket_factory = kw.pop('socket_factory', socket)
    if 'unix_socket' in kw:
      raise ValueError('%s does not support unix_sockets!' % self.__class__.__name__)
    TSSLSocket.__init__(self, *args, **kw)  # thrift does not support super()

  def __do_wrap(self, handshake_on_connect=False):
    # TODO(wickman) Thrift 0.9.1 supports keyfile and certfile.  File a
    # ticket to get delay_handshake added to the Thrift core libs.
    self.handle = ssl.wrap_socket(
        self.handle,
        ssl_version=self.SSL_VERSION,
        do_handshake_on_connect=handshake_on_connect,
        ca_certs=self.ca_certs,
        cert_reqs=self.cert_reqs)

  def open(self):
    try:
      resolved = self._resolveAddr()
      for offset, res in enumerate(resolved):
        sock_family, sock_type, _, _, ip_port = res[0:5]
        self.handle = self.__socket_factory.socket(sock_family, sock_type)
        self.handle.settimeout(self._timeout)
        reraise = (offset == len(resolved) - 1)

        if not self.__delay_handshake:
          self.__do_wrap(handshake_on_connect=True)

        try:
          self.handle.connect(ip_port)
        except self.__socket_factory.error:
          if reraise:
            raise

        if self.__delay_handshake:
          self.__do_wrap(handshake_on_connect=False)
          self.handle.do_handshake()

    except self.__socket_factory.error as e:
      message = 'Could not connect to %s:%d: %s' % (self.host, self.port, e)
      raise TTransportException(type=TTransportException.NOT_OPEN, message=message)

    if self.validate:
      self._validate_cert()



#
# Autogenerated by Thrift
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#

from thrift.Thrift import *

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None


class AnnotationType(object):
  BOOL = 0
  BYTES = 1
  I16 = 2
  I32 = 3
  I64 = 4
  DOUBLE = 5
  STRING = 6

  _VALUES_TO_NAMES = {
    0: "BOOL",
    1: "BYTES",
    2: "I16",
    3: "I32",
    4: "I64",
    5: "DOUBLE",
    6: "STRING",
  }

  _NAMES_TO_VALUES = {
    "BOOL": 0,
    "BYTES": 1,
    "I16": 2,
    "I32": 3,
    "I64": 4,
    "DOUBLE": 5,
    "STRING": 6,
  }


class Endpoint(object):
  """
  Attributes:
   - ipv4
   - port
   - service_name
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'ipv4', None, None, ), # 1
    (2, TType.I16, 'port', None, None, ), # 2
    (3, TType.STRING, 'service_name', None, None, ), # 3
  )

  def __init__(self, ipv4=None, port=None, service_name=None,):
    self.ipv4 = ipv4
    self.port = port
    self.service_name = service_name

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.ipv4 = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I16:
          self.port = iprot.readI16();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.service_name = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Endpoint')
    if self.ipv4 != None:
      oprot.writeFieldBegin('ipv4', TType.I32, 1)
      oprot.writeI32(self.ipv4)
      oprot.writeFieldEnd()
    if self.port != None:
      oprot.writeFieldBegin('port', TType.I16, 2)
      oprot.writeI16(self.port)
      oprot.writeFieldEnd()
    if self.service_name != None:
      oprot.writeFieldBegin('service_name', TType.STRING, 3)
      oprot.writeString(self.service_name)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Annotation(object):
  """
  Attributes:
   - timestamp
   - value
   - host
   - duration
  """

  thrift_spec = (
    None, # 0
    (1, TType.I64, 'timestamp', None, None, ), # 1
    (2, TType.STRING, 'value', None, None, ), # 2
    (3, TType.STRUCT, 'host', (Endpoint, Endpoint.thrift_spec), None, ), # 3
    (4, TType.I32, 'duration', None, None, ), # 4
  )

  def __init__(self, timestamp=None, value=None, host=None, duration=None,):
    self.timestamp = timestamp
    self.value = value
    self.host = host
    self.duration = duration

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I64:
          self.timestamp = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.value = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.host = Endpoint()
          self.host.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I32:
          self.duration = iprot.readI32();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Annotation')
    if self.timestamp != None:
      oprot.writeFieldBegin('timestamp', TType.I64, 1)
      oprot.writeI64(self.timestamp)
      oprot.writeFieldEnd()
    if self.value != None:
      oprot.writeFieldBegin('value', TType.STRING, 2)
      oprot.writeString(self.value)
      oprot.writeFieldEnd()
    if self.host != None:
      oprot.writeFieldBegin('host', TType.STRUCT, 3)
      self.host.write(oprot)
      oprot.writeFieldEnd()
    if self.duration != None:
      oprot.writeFieldBegin('duration', TType.I32, 4)
      oprot.writeI32(self.duration)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class BinaryAnnotation(object):
  """
  Attributes:
   - key
   - value
   - annotation_type
   - host
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'key', None, None, ), # 1
    (2, TType.STRING, 'value', None, None, ), # 2
    (3, TType.I32, 'annotation_type', None, None, ), # 3
    (4, TType.STRUCT, 'host', (Endpoint, Endpoint.thrift_spec), None, ), # 4
  )

  def __init__(self, key=None, value=None, annotation_type=None, host=None,):
    self.key = key
    self.value = value
    self.annotation_type = annotation_type
    self.host = host

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.key = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.value = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I32:
          self.annotation_type = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRUCT:
          self.host = Endpoint()
          self.host.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('BinaryAnnotation')
    if self.key != None:
      oprot.writeFieldBegin('key', TType.STRING, 1)
      oprot.writeString(self.key)
      oprot.writeFieldEnd()
    if self.value != None:
      oprot.writeFieldBegin('value', TType.STRING, 2)
      oprot.writeString(self.value)
      oprot.writeFieldEnd()
    if self.annotation_type != None:
      oprot.writeFieldBegin('annotation_type', TType.I32, 3)
      oprot.writeI32(self.annotation_type)
      oprot.writeFieldEnd()
    if self.host != None:
      oprot.writeFieldBegin('host', TType.STRUCT, 4)
      self.host.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Span(object):
  """
  Attributes:
   - trace_id
   - name
   - id
   - parent_id
   - annotations
   - binary_annotations
   - debug
  """

  thrift_spec = (
    None, # 0
    (1, TType.I64, 'trace_id', None, None, ), # 1
    None, # 2
    (3, TType.STRING, 'name', None, None, ), # 3
    (4, TType.I64, 'id', None, None, ), # 4
    (5, TType.I64, 'parent_id', None, None, ), # 5
    (6, TType.LIST, 'annotations', (TType.STRUCT,(Annotation, Annotation.thrift_spec)), None, ), # 6
    None, # 7
    (8, TType.LIST, 'binary_annotations', (TType.STRUCT,(BinaryAnnotation, BinaryAnnotation.thrift_spec)), None, ), # 8
    (9, TType.BOOL, 'debug', None, None, ), # 9
  )

  def __init__(self, trace_id=None, name=None, id=None, parent_id=None, annotations=None, binary_annotations=None, debug=None,):
    self.trace_id = trace_id
    self.name = name
    self.id = id
    self.parent_id = parent_id
    self.annotations = annotations
    self.binary_annotations = binary_annotations
    self.debug = debug

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I64:
          self.trace_id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.name = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I64:
          self.id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.I64:
          self.parent_id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.LIST:
          self.annotations = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = Annotation()
            _elem5.read(iprot)
            self.annotations.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.LIST:
          self.binary_annotations = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = BinaryAnnotation()
            _elem11.read(iprot)
            self.binary_annotations.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.BOOL:
          self.debug = iprot.readBool();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Span')
    if self.trace_id != None:
      oprot.writeFieldBegin('trace_id', TType.I64, 1)
      oprot.writeI64(self.trace_id)
      oprot.writeFieldEnd()
    if self.name != None:
      oprot.writeFieldBegin('name', TType.STRING, 3)
      oprot.writeString(self.name)
      oprot.writeFieldEnd()
    if self.id != None:
      oprot.writeFieldBegin('id', TType.I64, 4)
      oprot.writeI64(self.id)
      oprot.writeFieldEnd()
    if self.parent_id != None:
      oprot.writeFieldBegin('parent_id', TType.I64, 5)
      oprot.writeI64(self.parent_id)
      oprot.writeFieldEnd()
    if self.annotations != None:
      oprot.writeFieldBegin('annotations', TType.LIST, 6)
      oprot.writeListBegin(TType.STRUCT, len(self.annotations))
      for iter12 in self.annotations:
        iter12.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.binary_annotations != None:
      oprot.writeFieldBegin('binary_annotations', TType.LIST, 8)
      oprot.writeListBegin(TType.STRUCT, len(self.binary_annotations))
      for iter13 in self.binary_annotations:
        iter13.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.debug != None:
      oprot.writeFieldBegin('debug', TType.BOOL, 9)
      oprot.writeBool(self.debug)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ClientId(object):
  """
  At connection time, we can let the server know who we are so
  they can book keep and optionally reject unknown clients.

  Attributes:
   - name
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'name', None, None, ), # 1
  )

  def __init__(self, name=None,):
    self.name = name

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.name = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ClientId')
    if self.name != None:
      oprot.writeFieldBegin('name', TType.STRING, 1)
      oprot.writeString(self.name)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class RequestContext(object):
  """
  This struct serializes com.twitter.finagle.Context

  Attributes:
   - key
   - value
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'key', None, None, ), # 1
    (2, TType.STRING, 'value', None, None, ), # 2
  )

  def __init__(self, key=None, value=None,):
    self.key = key
    self.value = value

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.key = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.value = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('RequestContext')
    if self.key != None:
      oprot.writeFieldBegin('key', TType.STRING, 1)
      oprot.writeString(self.key)
      oprot.writeFieldEnd()
    if self.value != None:
      oprot.writeFieldBegin('value', TType.STRING, 2)
      oprot.writeString(self.value)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Delegation(object):
  """
  Serializes an individual delegation.

  Attributes:
   - src
   - dst
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'src', None, None, ), # 1
    (2, TType.STRING, 'dst', None, None, ), # 2
  )

  def __init__(self, src=None, dst=None,):
    self.src = src
    self.dst = dst

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.src = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.dst = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Delegation')
    if self.src != None:
      oprot.writeFieldBegin('src', TType.STRING, 1)
      oprot.writeString(self.src)
      oprot.writeFieldEnd()
    if self.dst != None:
      oprot.writeFieldBegin('dst', TType.STRING, 2)
      oprot.writeString(self.dst)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class RequestHeader(object):
  """
  RequestHeader defines headers for the request. These carry the span data, and
  a flag indicating whether the request is to be debugged.

  Attributes:
   - trace_id
   - span_id
   - parent_span_id
   - sampled
   - client_id
   - flags
   - contexts
   - dest
   - delegations
  """

  thrift_spec = (
    None, # 0
    (1, TType.I64, 'trace_id', None, None, ), # 1
    (2, TType.I64, 'span_id', None, None, ), # 2
    (3, TType.I64, 'parent_span_id', None, None, ), # 3
    None, # 4
    (5, TType.BOOL, 'sampled', None, None, ), # 5
    (6, TType.STRUCT, 'client_id', (ClientId, ClientId.thrift_spec), None, ), # 6
    (7, TType.I64, 'flags', None, None, ), # 7
    (8, TType.LIST, 'contexts', (TType.STRUCT,(RequestContext, RequestContext.thrift_spec)), None, ), # 8
    (9, TType.STRING, 'dest', None, None, ), # 9
    (10, TType.LIST, 'delegations', (TType.STRUCT,(Delegation, Delegation.thrift_spec)), None, ), # 10
  )

  def __init__(self, trace_id=None, span_id=None, parent_span_id=None, sampled=None, client_id=None, flags=None, contexts=None, dest=None, delegations=None,):
    self.trace_id = trace_id
    self.span_id = span_id
    self.parent_span_id = parent_span_id
    self.sampled = sampled
    self.client_id = client_id
    self.flags = flags
    self.contexts = contexts
    self.dest = dest
    self.delegations = delegations

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I64:
          self.trace_id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.span_id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.parent_span_id = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.BOOL:
          self.sampled = iprot.readBool();
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRUCT:
          self.client_id = ClientId()
          self.client_id.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.I64:
          self.flags = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.LIST:
          self.contexts = []
          (_etype17, _size14) = iprot.readListBegin()
          for _i18 in xrange(_size14):
            _elem19 = RequestContext()
            _elem19.read(iprot)
            self.contexts.append(_elem19)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.STRING:
          self.dest = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 10:
        if ftype == TType.LIST:
          self.delegations = []
          (_etype23, _size20) = iprot.readListBegin()
          for _i24 in xrange(_size20):
            _elem25 = Delegation()
            _elem25.read(iprot)
            self.delegations.append(_elem25)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('RequestHeader')
    if self.trace_id != None:
      oprot.writeFieldBegin('trace_id', TType.I64, 1)
      oprot.writeI64(self.trace_id)
      oprot.writeFieldEnd()
    if self.span_id != None:
      oprot.writeFieldBegin('span_id', TType.I64, 2)
      oprot.writeI64(self.span_id)
      oprot.writeFieldEnd()
    if self.parent_span_id != None:
      oprot.writeFieldBegin('parent_span_id', TType.I64, 3)
      oprot.writeI64(self.parent_span_id)
      oprot.writeFieldEnd()
    if self.sampled != None:
      oprot.writeFieldBegin('sampled', TType.BOOL, 5)
      oprot.writeBool(self.sampled)
      oprot.writeFieldEnd()
    if self.client_id != None:
      oprot.writeFieldBegin('client_id', TType.STRUCT, 6)
      self.client_id.write(oprot)
      oprot.writeFieldEnd()
    if self.flags != None:
      oprot.writeFieldBegin('flags', TType.I64, 7)
      oprot.writeI64(self.flags)
      oprot.writeFieldEnd()
    if self.contexts != None:
      oprot.writeFieldBegin('contexts', TType.LIST, 8)
      oprot.writeListBegin(TType.STRUCT, len(self.contexts))
      for iter26 in self.contexts:
        iter26.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.dest != None:
      oprot.writeFieldBegin('dest', TType.STRING, 9)
      oprot.writeString(self.dest)
      oprot.writeFieldEnd()
    if self.delegations != None:
      oprot.writeFieldBegin('delegations', TType.LIST, 10)
      oprot.writeListBegin(TType.STRUCT, len(self.delegations))
      for iter27 in self.delegations:
        iter27.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ResponseHeader(object):
  """
  The Response carries a reply header for tracing. These are
  empty unless the request is being debugged, in which case a
  transcript is copied.

  Attributes:
   - spans
   - contexts
  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'spans', (TType.STRUCT,(Span, Span.thrift_spec)), None, ), # 1
    (2, TType.LIST, 'contexts', (TType.STRUCT,(RequestContext, RequestContext.thrift_spec)), None, ), # 2
  )

  def __init__(self, spans=None, contexts=None,):
    self.spans = spans
    self.contexts = contexts

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.spans = []
          (_etype31, _size28) = iprot.readListBegin()
          for _i32 in xrange(_size28):
            _elem33 = Span()
            _elem33.read(iprot)
            self.spans.append(_elem33)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.contexts = []
          (_etype37, _size34) = iprot.readListBegin()
          for _i38 in xrange(_size34):
            _elem39 = RequestContext()
            _elem39.read(iprot)
            self.contexts.append(_elem39)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ResponseHeader')
    if self.spans != None:
      oprot.writeFieldBegin('spans', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.spans))
      for iter40 in self.spans:
        iter40.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.contexts != None:
      oprot.writeFieldBegin('contexts', TType.LIST, 2)
      oprot.writeListBegin(TType.STRUCT, len(self.contexts))
      for iter41 in self.contexts:
        iter41.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ConnectionOptions(object):
  """
  These are connection-level options negotiated during protocol
  upgrade.
  """

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ConnectionOptions')
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class UpgradeReply(object):
  """
  This is the struct that a successful upgrade will reply with.
  """

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('UpgradeReply')
    oprot.writeFieldStop()
    oprot.writeStructEnd()
    def validate(self):
      return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#
# Autogenerated by Thrift
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#

from thrift.Thrift import *
from ttypes import *

CLIENT_SEND = "cs"
CLIENT_RECV = "cr"
SERVER_SEND = "ss"
SERVER_RECV = "sr"
SERVER_ADDR = "sa"
CLIENT_ADDR = "ca"



# ==================================================================================================
# Copyright 2013 Twitter, Inc.
# --------------------------------------------------------------------------------------------------
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==================================================================================================

import functools
import threading

from thrift.Thrift import TApplicationException, TMessageType
from thrift.protocol import TBinaryProtocol

from gen.twitter.finagle.thrift.ttypes import (
    ClientId,
    ConnectionOptions,
    RequestHeader,
    ResponseHeader,
    UpgradeReply,
)

from .trace import Trace


def upgrade_protocol_to_finagle(protocol):
  UPGRADE_METHOD = "__can__finagle__trace__v3__"

  def send(protocol):
    protocol.writeMessageBegin(UPGRADE_METHOD, TMessageType.CALL, 0)
    connection_options = ConnectionOptions()
    connection_options.write(protocol)
    protocol.writeMessageEnd()
    protocol.trans.flush()

  def recv(protocol):
    (fname, mtype, rseqid) = protocol.readMessageBegin()
    if fname != UPGRADE_METHOD:
      raise Exception('Unexpected error upgrading Finagle transport!')
    if mtype == TMessageType.EXCEPTION:
      exc = TApplicationException()
      exc.read(protocol)
      protocol.readMessageEnd()
      raise exc
    reply = UpgradeReply()
    reply.read(protocol)
    protocol.readMessageEnd()
    return reply

  send(protocol)
  return recv(protocol)


class TFinagleProtocol(TBinaryProtocol.TBinaryProtocolAccelerated):
  def __init__(self, *args, **kw):
    self._locals = threading.local()
    self._finagle_upgraded = False
    self._client_id = kw.pop('client_id', None)
    self._client_id = ClientId(name=self._client_id) if self._client_id else None
    TBinaryProtocol.TBinaryProtocolAccelerated.__init__(self, *args, **kw)
    try:
      upgrade_protocol_to_finagle(self)
      self._finagle_upgraded = True
    except TApplicationException:
      pass

  def to_request_header(self, trace_id):
    return RequestHeader(trace_id=trace_id.trace_id.value,
                         parent_span_id=trace_id.parent_id.value,
                         span_id=trace_id.span_id.value,
                         sampled=trace_id.sampled,
                         client_id=self._client_id)

  def writeMessageBegin(self, *args, **kwargs):
    if self._finagle_upgraded:
      if not hasattr(self._locals, 'trace'):
        self._locals.trace = Trace()
      trace_id = self._locals.trace.get()
      self.to_request_header(trace_id).write(self)
      with self._locals.trace.push(trace_id):
        return TBinaryProtocol.TBinaryProtocolAccelerated.writeMessageBegin(self, *args, **kwargs)
    else:
      return TBinaryProtocol.TBinaryProtocolAccelerated.writeMessageBegin(self, *args, **kwargs)

  def readMessageBegin(self, *args, **kwargs):
    if self._finagle_upgraded:
      header = ResponseHeader()
      header.read(self)
      self._locals.last_response = header
    return TBinaryProtocol.TBinaryProtocolAccelerated.readMessageBegin(self, *args, **kwargs)


def TFinagleProtocolWithClientId(client_id):
  return functools.partial(TFinagleProtocol, client_id=client_id)



from twitter.common.lang import Compatibility

class Address(object):
  class InvalidFormat(Exception):
    pass

  @staticmethod
  def sanity_check(host, port):
    if not isinstance(host, Compatibility.string):
      raise Address.InvalidFormat('Host must be a string, got %s' % host)
    if not isinstance(port, (int, long)):
      raise Address.InvalidFormat('Port must be an integer, got %s' % port)
    if port <= 0:
      raise Address.InvalidFormat('Port must be a positive integer, got %s' % port)

  @staticmethod
  def from_string(*args, **kw):
    if (kw or len(args) != 1 or not isinstance(args[0], Compatibility.string)
        or not len(args[0].split(':')) == 2):
      raise Address.InvalidFormat('from_string expects "host:port" string.')
    host, port = args[0].split(':')
    try:
      port = int(port)
    except ValueError:
      raise Address.InvalidFormat('Port must be an integer, got %s' % port)
    Address.sanity_check(host, port)
    return Address(host, port)

  @staticmethod
  def from_pair(*args, **kw):
    if (kw or len(args) != 2 or not isinstance(args[0], Compatibility.string)
        or not isinstance(args[1], (int, long))):
      raise Address.InvalidFormat('from_pair expects host, port as input!')
    Address.sanity_check(args[0], args[1])
    return Address(args[0], args[1])

  @staticmethod
  def from_tuple(*args, **kw):
    if kw or len(args) != 1 or len(args[0]) != 2:
      raise Address.InvalidFormat('from_tuple expects (host, port) tuple as input!')
    host, port = args[0]
    Address.sanity_check(host, port)
    return Address(host, port)

  @staticmethod
  def from_address(*args, **kw):
    if kw or len(args) != 1 or not isinstance(args[0], Address):
      raise Address.InvalidFormat('from_address expects an address as input!')
    return Address(args[0].host, args[0].port)

  @staticmethod
  def parse(*args, **kw):
    for parser in [Address.from_string, Address.from_pair, Address.from_address, Address.from_tuple]:
      try:
        return parser(*args, **kw)
      except Address.InvalidFormat:
        continue
    raise Address.InvalidFormat('Could not parse input: args=%s kw=%s' % (
      repr(args), repr(kw)))

  def __init__(self, host, port):
    self._host = host
    self._port = port

  @property
  def host(self):
    return self._host

  @property
  def port(self):
    return self._port



# ==================================================================================================
# Copyright 2013 Twitter, Inc.
# --------------------------------------------------------------------------------------------------
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==================================================================================================

import copy
from contextlib import contextmanager
import random
import re

class SpanId(object):
  __slots__ = ('_value',)

  HEX_REGEX = re.compile(r'^[a-f0-9]{16}$', re.IGNORECASE)

  class InvalidSpanId(ValueError):
    def __init__(self, value):
      ValueError.__init__(self, 'Invalid SpanId: %s' % repr(value))

  @staticmethod
  def from_value(value):
    if isinstance(value, str):
      if SpanId.HEX_REGEX.match(value):
        return SpanId(int(value, 16))
    elif isinstance(value, (int, long)):
      return SpanId(value)
    elif isinstance(value, SpanId):
      return SpanId(value.value)
    elif value is None:
      return SpanId(None)
    raise SpanId.InvalidSpanId(value)

  def __init__(self, value):
    self._value = value

  @property
  def value(self):
    return self._value

  def __str__(self):
    return 'SpanId(%016x)' % (self._value if self._value is not None else 'Empty')


class TraceId(object):
  @staticmethod
  def rand():
    return random.randint(0, 2**63-1)

  def __init__(self, trace_id, parent_id, span_id, sampled):
    self.trace_id = SpanId.from_value(trace_id)
    self.parent_id = SpanId.from_value(parent_id)
    self.span_id = SpanId.from_value(span_id)
    self.sampled = bool(sampled)

  def next(self):
    return TraceId(self.trace_id, self.span_id, TraceId.rand(), self.sampled)

  def __str__(self):
    return 'TraceId(trace_id = %s, parent_id = %s, span_id = %s, sampled = %s)' % (
      self.trace_id, self.parent_id, self.span_id, self.sampled)


class Trace(object):
  """
    The container of a trace.  Typically stored as a threadlocal on each
    finagle-upgraded protocol.
  """
  def __init__(self, sample_rate=0.001):
    assert 0.0 <= sample_rate <= 1.0
    self._sample_rate = sample_rate
    self._stack = []

  def get(self):
    if len(self._stack) == 0:
      span_id = TraceId.rand()
      trace_id = TraceId(span_id, None, span_id, self.should_sample())
      self._stack.append(trace_id)
    return self._stack[-1]

  @contextmanager
  def push(self, trace_id):
    self._stack.append(trace_id)
    try:
      yield self
    finally:
      self._stack.pop()

  @contextmanager
  def unwind(self):
    trace_id_copy = copy.deepcopy(self._stack[-1])
    try:
      yield self
    finally:
      self._stack[-1] = trace_id_copy

  def pop(self):
    return self._stack.pop()

  def should_sample(self):
    return random.random() < self._sample_rate





import functools
from thrift.transport import TSocket, TTransport
from thrift.protocol import TBinaryProtocol
from twitter.common.rpc.address import Address


class ConnectionClosable(object):
  """Mixin class for thrift connection closability."""

  def close(self):
    if hasattr(self, '_connection') and self._connection:
      self._connection.close()
    return

  def __enter__(self):
    return self

  def __exit__(self, *args, **kwargs):
    self.close()
    return False


class ConnectionFactory(object):
  def __init__(self, connection_klazz):
    self._connection_klazz = connection_klazz

  def __call__(self, *args, **kw):
    address = Address.parse(*args, **kw)
    return self._connection_klazz(host=address.host, port=address.port)


class TransportFactory(object):
  def __init__(self, transport_klazz):
    self._transport_klazz = transport_klazz

  def __call__(self, connection):
    return self._transport_klazz(connection)


class ProtocolFactory(object):
  def __init__(self, protocol_klazz):
    self._protocol_klazz = protocol_klazz

  def __call__(self, transport):
    return self._protocol_klazz(transport)


class ClientFactory(object):
  def __init__(self, client_iface, connection):
    self._client_iface = client_iface
    self._connection = connection

  def _mixin(self, klazz, mixin):
    if mixin not in klazz.__bases__:
      klazz.__bases__ += (mixin,)
    return klazz

  def __call__(self, protocol):
    # Mix ConnectionClosable class into the client class.
    client_class = getattr(self._client_iface, 'Client')
    client_class = self._mixin(client_class, ConnectionClosable)

    # Instantiate and set _connection object on the instance.
    client = client_class(protocol)
    client._connection = self._connection

    return client


def make_client(client_iface, *args, **kw):
  """
    Ex, basic usage:
      make_client(UserService, 'localhost', 9999)

    Ex, make an SSL socket server (see also make_server)
      make_client(UserService, 'smf1-amk-25-sr1.prod.twitter.com', 9999,
                  connection=TSocket.TSSLServerSocket,
                  ca_certs=...)

    Ex, make a finagle client:
      make_client(UserService, 'localhost', 9999,
                  protocol=TFinagleProtocol)

    And one with a client_id
      make_client(UserService, 'localhost', 9999,
                  protocol=functools.partial(TFinagleProtocol, client_id="test_client"))

    (this is equivalent to
      make_client(UserService, 'localhost', 9999,
                  protocol=TFinagleProtocolWithClientId("test_client")))

    Ex, bind to a unix_socket instead of host/port pair (unix_socket is kwarg to
    TSocket.TSocket):
      make_client(UserService, unix_socket=...opened-fifo...)

    N.B. This can also be used as a contextmanager or with contextlib.closing to
         automatically handle closing of the thrift connection (or manually via close()).

      with make_client(...) as c:
        c.somefunc()
  """
  protocol_class = kw.pop('protocol', TBinaryProtocol.TBinaryProtocolAccelerated)
  transport_class = kw.pop('transport', TTransport.TFramedTransport)
  connection_class = kw.pop('connection', TSocket.TSocket)

  connection = ConnectionFactory(connection_class)(*args, **kw)
  connection.open()

  return ClientFactory(client_iface, connection)(
           ProtocolFactory(protocol_class)(
             TransportFactory(transport_class)(connection)))


def make_client_factory(client_iface):
  return functools.partial(make_client, client_iface)




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(src/python/twitter/common/rpc/BUILD:rpc-packaged)

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python'],
    'description': 'twitter.common thrift helpers including finagle and ssl transports.',
    'install_requires': [   'twitter.common.lang==0.1.0',
                            'thrift==0.9.0',
                            'thrift'],
    'license': 'Apache License, Version 2.0',
    'name': 'twitter.common.rpc',
    'namespace_packages': [   'gen',
                              'gen.twitter',
                              'gen.twitter.finagle',
                              'twitter',
                              'twitter.common'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   '.',
                    'gen',
                    'gen.twitter',
                    'gen.twitter.finagle',
                    'gen.twitter.finagle.thrift',
                    'twitter',
                    'twitter.common',
                    'twitter.common.rpc',
                    'twitter.common.rpc.finagle',
                    'twitter.common.rpc.transports'],
    'url': 'https://github.com/twitter/commons',
    'version': '0.1.0',
    'zip_safe': True}
)
